@@ -16,9 +16,9 @@ class ExampleData(JHUData):
     Args:
         clean_df (pandas.DataFrame or None): cleaned data
 
-            Index:
+            Index
                 - reset index
-            Columns:
+            Columns
                 - Date (pd.TimeStamp): Observation date
                 - Country (pandas.Category): country/region name
                 - Province (pandas.Category): province/prefecture/sstate name
@@ -34,15 +34,16 @@ class ExampleData(JHUData):
     def __init__(self, clean_df=None, tau=1440, start_date="22Jan2020"):
         if clean_df is None:
             clean_df = pd.DataFrame(columns=self.COLUMNS)
-        clean_df = self.ensure_dataframe(
+        clean_df = self._ensure_dataframe(
             clean_df, name="clean_df", columns=self.COLUMNS)
         self._raw = clean_df.copy()
         self._cleaned_df = clean_df.copy()
         self._citation = str()
-        self.tau = self.ensure_tau(tau)
-        self.start_date = self.ensure_date(start_date, name="start_date")
+        self.tau = self._ensure_tau(tau)
+        self.start_date = self._ensure_date(start_date, name="start_date")
         self._specialized_dict = {}
         self.nondim_dict = {}
+        self._recovery_period = None
 
     def _model_to_area(self, model=None, country=None, province=None):
         """
@@ -64,7 +65,7 @@ class ExampleData(JHUData):
             return (country, province)
         if model is None:
             raise ValueError("@model or @country must be specified.")
-        model = self.ensure_subclass(model, ModelBase, name="model")
+        model = self._ensure_subclass(model, ModelBase, name="model")
         return (model.NAME, province)
 
     def add(self, model, country=None, province=None, **kwargs):
@@ -79,12 +80,12 @@ class ExampleData(JHUData):
             province (str or None): province name
             kwargs: the other keyword arguments of ODESimulator.add()
 
-        Notes:
+        Note:
             If country is None, the name of the model will be used.
             If province is None, '-' will be used.
         """
         # Arguments
-        model = self.ensure_subclass(model, ModelBase, name="model")
+        model = self._ensure_subclass(model, ModelBase, name="model")
         arg_dict = model.EXAMPLE.copy()
         arg_dict.update(kwargs)
         population = arg_dict["population"]
@@ -139,7 +140,7 @@ class ExampleData(JHUData):
             country (str or None): country name
             province (str or None): province name
 
-        Notes:
+        Note:
             If country is None, the name of the model will be used.
             If province is None, '-' will be used.
         """
@@ -159,7 +160,7 @@ class ExampleData(JHUData):
             country (str or None): country name
             province (str or None): province name
 
-        Notes:
+        Note:
             If country is None, the name of the model will be used.
             If province is None, '-' will be used.
         """
@@ -182,9 +183,9 @@ class ExampleData(JHUData):
 
         Returns:
             (pandas.DataFrame)
-                Index:
+                Index
                     reset index
-                Columns:
+                Columns
                     - Date (pd.TimeStamp): Observation date
                     - Confirmed (int): the number of confirmed cases
                     - Infected (int): the number of currently infected cases
@@ -192,7 +193,7 @@ class ExampleData(JHUData):
                     - Recovered (int): the number of recovered cases (> 0)
                     - Susceptible (int): the number of susceptible cases, if calculated
 
-        Notes:
+        Note:
             If country is None, the name of the model will be used.
             If province is None, '-' will be used.
             If @population is not None, the number of susceptible cases will be calculated.
