@@ -5,7 +5,6 @@ from pathlib import Path
 import numpy as np
 import pandas as pd
 from dask import dataframe as dd
-import swifter
 from covsirphy.util.plotting import line_plot
 from covsirphy.util.error import PCRIncorrectPreconditionError, SubsetNotFoundError
 from covsirphy.cleaning.cbase import CleaningBase
@@ -40,14 +39,12 @@ class PCRData(CleaningBase):
                 filename, dtype={"Province/State": "object"}
             ).compute()
             self._cleaned_df = self._cleaning()
-        self.interval = self.ensure_natural_int(interval, name="interval")
-        self.min_pcr_tests = self.ensure_natural_int(
+        self.interval = self._ensure_natural_int(interval, name="interval")
+        self.min_pcr_tests = self._ensure_natural_int(
             min_pcr_tests, name="min_pcr_tests")
         self._citation = citation or ""
         # Cleaned dataset of "Our World In Data"
         self._cleaned_df_owid = pd.DataFrame()
-        # To avoid "imported but unused"
-        self.__swifter = swifter
 
     def cleaned(self):
         """
@@ -55,16 +52,16 @@ class PCRData(CleaningBase):
 
         Returns:
             pandas.DataFrame
-                Index:
+                Index
                     reset index
-                Columns:
+                Columns
                     - Date (pd.TimeStamp): Observation date
                     - Country (pandas.Category): country/region name
                     - Province (pandas.Category): province/prefecture/state name
                     - Tests (int): the number of total tests performed
                     - Confirmed (int): the number of confirmed cases
 
-        Notes:
+        Note:
             Cleaning method is defined by self._cleaning() method.
         """
         return self._cleaned_df.loc[:, self.PCR_COLUMNS]
@@ -76,9 +73,9 @@ class PCRData(CleaningBase):
 
         Returns:
             pandas.DataFrame
-                Index:
+                Index
                     reset index
-                Columns:
+                Columns
                     - Date (pd.TimeStamp): Observation date
                     - ISO3 (str): ISO3 code
                     - Country (pandas.Category): country/region name
@@ -97,7 +94,7 @@ class PCRData(CleaningBase):
             axis=1
         )
         # Confirm the expected columns are in raw data
-        self.ensure_dataframe(
+        self._ensure_dataframe(
             df, name="the raw data", columns=self.PCR_COLUMNS)
         # Datetime columns
         df[self.DATE] = pd.to_datetime(df[self.DATE])
@@ -137,7 +134,7 @@ class PCRData(CleaningBase):
             covsirphy.PCRData: PCR dataset
         """
         instance = cls(filename=None)
-        instance._cleaned_df = cls.ensure_dataframe(
+        instance._cleaned_df = cls._ensure_dataframe(
             dataframe, name="dataframe", columns=cls.PCR_COLUMNS)
         return instance
 
@@ -169,7 +166,7 @@ class PCRData(CleaningBase):
             (df[self.TESTS].isna()) & (df[self.DATE] == df[self.DATE].max())]
         re_countries_set = set(na_last_df[self.ISO3].unique())
         df["cumsum"] = df.groupby(self.ISO3)[self.T_DIFF].cumsum()
-        df[self.TESTS] = df[[self.ISO3, self.TESTS, "cumsum"]].swifter.progress_bar(False).apply(
+        df[self.TESTS] = df[[self.ISO3, self.TESTS, "cumsum"]].apply(
             lambda x: x[1] if x[0] in re_countries_set else x[2], axis=1)
         df = df.drop("cumsum", axis=1)
         # Drop duplicated records
@@ -227,9 +224,11 @@ class PCRData(CleaningBase):
 
         Args:
             country_data (covsirphy.CountryData): dataset object of the country
-                Index: reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
+
+                Index
+                    reset index
+                Columns
+                    - Date (pandas.TimeStamp): Observation date
                     - Province (pandas.Category): province name
                     - Tests (int): the number of total tests performed
                     - Confirmed (int): the number of confirmed cases
@@ -238,10 +237,10 @@ class PCRData(CleaningBase):
         Returns:
             covsirphy.PCRData: self
         """
-        self.ensure_instance(country_data, CountryData, name="country_data")
+        self._ensure_instance(country_data, CountryData, name="country_data")
         # Read new dataset
         country = country_data.country
-        new = self.ensure_dataframe(
+        new = self._ensure_dataframe(
             country_data.cleaned(), name="the raw data", columns=self.PCR_COLUMNS)
         new = new.loc[:, self.PCR_COLUMNS]
         new[self.ISO3] = self.country_to_iso3(country)
@@ -264,14 +263,18 @@ class PCRData(CleaningBase):
 
         Args:
             df (pandas.DataFrame):
-                Index: Date (pandas.TimeStamp)
-                Columns: Tests, Confirmed
+                Index
+                    Date (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed
             variable (str): variable name to show monotonic increasing
 
         Returns:
             pandas.DataFrame: complemented records
-                Index: Date (pandas.TimeStamp)
-                Columns: Tests, Confirmed
+                Index
+                    Date (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed
         """
         # Whether complement is necessary or not
         if df[variable].is_monotonic_increasing:
@@ -298,8 +301,10 @@ class PCRData(CleaningBase):
 
         Args:
             df (pandas.DataFrame):
-                Index: Date (pandas.TimeStamp)
-                Columns: Tests, Confirmed, Tests_diff, C_diff
+                Index
+                    Sate (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed, Tests_diff, C_diff
             variable: the desired column to use
 
         Returns:
@@ -308,6 +313,53 @@ class PCRData(CleaningBase):
         max_frequency = df[variable].value_counts().max()
         return max_frequency > self.interval or not df.loc[df.index[-1], variable]
 
+    def _pcr_partial_complement_ending(self, df, window):
+        """
+        If ending test values do not change daily, while there are new cases,
+        apply previous diff() only to these ending unupdated values
+        and keep the previous valid ones.
+
+        Args:
+            df (pandas.DataFrame):
+                Index
+                    Date (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed
+            window (int): window of moving average, >= 1
+
+        Returns:
+            pandas.DataFrame: complemented test records
+                Index
+                    Date (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed
+        """
+        # Whether complement is necessary or not
+        tests_max = df[self.TESTS].max()
+        check_tests_ending = (
+            df[self.TESTS] == tests_max).sum() > self.interval
+        last_new_C = df[self.C].diff().rolling(window).mean().iloc[-1]
+        check_C = last_new_C > self.min_pcr_tests
+        if not (check_tests_ending and check_C):
+            return df
+        # Complement any ending unupdated test values
+        # that are not updated daily, by keeping and
+        # propagating forward previous valid diff()
+        # min_index: index for first ending max test reoccurrence
+        min_index = df[self.TESTS].idxmax() + 1
+        first_value = df.loc[min_index, self.TESTS]
+        df_ending = df.copy()
+        df_ending.loc[df_ending.duplicated(
+            [self.TESTS], keep="first"), self.TESTS] = None
+        diff_series = df_ending[self.TESTS].diff(
+        ).ffill().fillna(0).astype(np.int64)
+        diff_series.loc[diff_series.duplicated(keep="last")] = None
+        diff_series.interpolate(
+            method="linear", inplace=True, limit_direction="both")
+        df.loc[min_index:, self.TESTS] = first_value + \
+            diff_series.loc[min_index:].cumsum()
+        return df
+
     def _pcr_partial_complement(self, before_df, variable):
         """
         If there are missing values in variable column,
@@ -315,16 +367,20 @@ class PCRData(CleaningBase):
 
         Args:
             before_df (pandas.DataFrame):
-                Index: Date (pandas.TimeStamp)
-                Columns: Tests, Confirmed, Tests_diff, C_diff
+                Index
+                    Date (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed, Tests_diff, C_diff
             variable: the desired column to use
 
         Returns:
             pandas.DataFrame: complemented records
-                Index: Date (pandas.TimeStamp)
-                Columns: Tests, Confirmed, Tests_diff, C_diff
+                Index
+                    Date (pandas.TimeStamp)
+                Columns
+                    Tests, Confirmed, Tests_diff, C_diff
 
-        Notes:
+        Note:
             Filling NA with 0 will be always applied.
         """
         df = before_df.copy()
@@ -345,15 +401,16 @@ class PCRData(CleaningBase):
 
         Args:
             before_df (pandas.DataFrame):
-                Index: reset index
-                Columns: Date (pandas.TimeStamp), Tests, Confirmed
+                Index reset index
+                Columns Date (pandas.TimeStamp), Tests, Confirmed
             window (int): window of moving average, >= 1
 
         Returns:
             tuple (pandas.DataFrame, bool):
                 pandas.DataFrame
-                    Index: reset index
-                    Columns:
+                    Index
+                        reset index
+                    Columns
                         - Date (pd.TimeStamp): Observation date
                         - Tests (int): the number of total tests performed
                         - Confirmed (int): the number of confirmed cases
@@ -363,14 +420,17 @@ class PCRData(CleaningBase):
         """
         df = before_df.copy()
         df[self.TESTS].fillna(method="ffill", inplace=True)
-        if self.T_DIFF in df.columns:
-            df[self.T_DIFF].fillna(method="ffill", inplace=True)
         # Confirmed must show monotonic increasing
         df = self._pcr_monotonic(df, self.C)
         df = self._pcr_partial_complement(df, self.TESTS)
         # If Tests values are all valid, with no missing values in-between,
         # they must be monotonically increasing as well
+        compare_df = df.copy()
         df = self._pcr_monotonic(df, self.TESTS)
+        # Complement any ending unupdated test records
+        df = self._pcr_partial_complement_ending(df, window)
+        # Complemented or not
+        is_complemented = not df.equals(compare_df)
         # Calculate daily values for tests and confirmed (with window=1)
         df[self.T_DIFF] = df[self.TESTS].diff()
         df[self.C_DIFF] = df[self.C].diff()
@@ -394,9 +454,6 @@ class PCRData(CleaningBase):
         df = df.replace(0, np.nan)
         non_zero_index_start = df[self.T_DIFF].first_valid_index()
         df = df.loc[non_zero_index_start:].reset_index(drop=True)
-        # Complemented or not
-        is_complemented = df.drop(
-            [self.T_DIFF, self.C_DIFF], axis=1).equals(before_df)
         return (df, is_complemented)
 
     def _pcr_check_preconditions(self, df):
@@ -405,8 +462,8 @@ class PCRData(CleaningBase):
 
         Args:
             df (pandas.DataFrame):
-                Index: Date (pandas.TimeStamp)
-                Columns: Tests, Confirmed
+                Index Date (pandas.TimeStamp)
+                Columns Tests, Confirmed
 
         Return:
             bool: whether the dataset has sufficient data or not
@@ -480,23 +537,25 @@ class PCRData(CleaningBase):
             filename (str): filename of the figure, or None (display figure)
 
         Raises:
-            PCRIncorrectPreconditionError: the dataset has too many missing values
+            covsirphy.PCRIncorrectPreconditionError: the dataset has too many missing values
 
         Returns:
             pandas.DataFrame
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
+                Index
+                    reset index
+                Columns
+                    - Date (pandas.TimeStamp): Observation date
                     - Tests (int): the number of total tests performed
                     - Confirmed (int): the number of confirmed cases
                     - Tests_diff (int): daily tests performed
                     - Confirmed_diff (int): daily confirmed cases
                     - Test_positive_rate (float): positive rate (%) of the daily cases over the total daily tests performed
 
-        Notes:
+        Note:
             If non monotonic records were found for either confirmed cases or tests,
             "with partially complemented tests data" will be added to the title of the figure.
         """
-        window = self.ensure_natural_int(window, name="window")
+        window = self._ensure_natural_int(window, name="window")
         # Subset with area
         country_alias = self.ensure_country_name(country)
         province = province or self.UNKNOWN
@@ -508,7 +567,7 @@ class PCRData(CleaningBase):
         # Process PCR data
         df, is_complemented = self._pcr_processing(subset_df, window)
         # Calculate PCR values
-        df[self.PCR_RATE] = df[[self.C_DIFF, self.T_DIFF]].swifter.progress_bar(False).apply(
+        df[self.PCR_RATE] = df[[self.C_DIFF, self.T_DIFF]].apply(
             lambda x: x[0] / x[1] * 100 if x[1] > self.min_pcr_tests else 0, axis=1)
         if not show_figure:
             return df
@@ -538,8 +597,9 @@ class PCRData(CleaningBase):
 
         Returns:
             pandas.DataFrame
-                Index: reset index
-                Columns:
+                Index
+                    reset index
+                Columns
                     - Date (pd.TimeStamp): Observation date
                     - Tests (int): the number of total tests performed
                     - Confirmed (int): the number of confirmed cases
